@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -39,17 +39,18 @@ import './tab.js';
 import './ublock.js';
 import './utils.js';
 
-import cacheStorage from './cachestorage.js';
-import contextMenu from './contextmenu.js';
 import io from './assets.js';
-import lz4Codec from './lz4.js';
-import staticExtFilteringEngine from './static-ext-filtering.js';
-import staticFilteringReverseLookup from './reverselookup.js';
-import staticNetFilteringEngine from './static-net-filtering.js';
 import µb from './background.js';
-import webRequest from './traffic.js';
-import { redirectEngine } from './redirect-engine.js';
+import { filteringBehaviorChanged } from './broadcast.js';
+import cacheStorage from './cachestorage.js';
 import { ubolog } from './console.js';
+import contextMenu from './contextmenu.js';
+import lz4Codec from './lz4.js';
+import { redirectEngine } from './redirect-engine.js';
+import staticFilteringReverseLookup from './reverselookup.js';
+import staticExtFilteringEngine from './static-ext-filtering.js';
+import staticNetFilteringEngine from './static-net-filtering.js';
+import webRequest from './traffic.js';
 
 import {
     permanentFirewall,
@@ -62,10 +63,8 @@ import {
 
 /******************************************************************************/
 
-// Load all: executed once.
-
-(async ( ) => {
-// >>>>> start of private scope
+let lastVersionInt = 0;
+let thisVersionInt = 0;
 
 /******************************************************************************/
 
@@ -82,6 +81,10 @@ vAPI.app.onShutdown = ( ) => {
     permanentSwitches.reset();
 };
 
+vAPI.alarms.onAlarm.addListener(alarm => {
+    µb.alarmQueue.push(alarm.name);
+});
+
 /******************************************************************************/
 
 // This is called only once, when everything has been loaded in memory after
@@ -145,22 +148,29 @@ const initializeTabs = async ( ) => {
 // https://www.reddit.com/r/uBlockOrigin/comments/s7c9go/
 //   Abort suspending network requests when uBO is merely being installed.
 
-const onVersionReady = lastVersion => {
-    if ( lastVersion === vAPI.app.version ) { return; }
+const onVersionReady = async lastVersion => {
+    lastVersionInt = vAPI.app.intFromVersion(lastVersion);
+    thisVersionInt = vAPI.app.intFromVersion(vAPI.app.version);
+    if ( thisVersionInt === lastVersionInt ) { return; }
 
     vAPI.storage.set({
         version: vAPI.app.version,
         versionUpdateTime: Date.now(),
     });
 
-    const lastVersionInt = vAPI.app.intFromVersion(lastVersion);
-
     // Special case: first installation
     if ( lastVersionInt === 0 ) {
         vAPI.net.unsuspend({ all: true, discard: true });
         return;
     }
 
+    // Remove cache items with obsolete names
+    if ( lastVersionInt < vAPI.app.intFromVersion('1.56.1b5') ) {
+        io.remove(`compiled/${µb.pslAssetKey}`);
+        io.remove('compiled/redirectEngine/resources');
+        io.remove('selfie/main');
+    }
+
     // Since built-in resources may have changed since last version, we
     // force a reload of all resources.
     redirectEngine.invalidateResourcesSelfie(io);
@@ -168,11 +178,6 @@ const onVersionReady = lastVersion => {
 
 /******************************************************************************/
 
-// https://github.com/chrisaljoudi/uBlock/issues/226
-// Whitelist in memory.
-// Whitelist parser needs PSL to be ready.
-// gorhill 2014-12-15: not anymore
-//
 // https://github.com/uBlockOrigin/uBlock-issues/issues/1433
 //   Allow admins to add their own trusted-site directives.
 
@@ -180,16 +185,38 @@ const onNetWhitelistReady = (netWhitelistRaw, adminExtra) => {
     if ( typeof netWhitelistRaw === 'string' ) {
         netWhitelistRaw = netWhitelistRaw.split('\n');
     }
-    // Append admin-controlled trusted-site directives
-    if (
-        adminExtra instanceof Object &&
-        Array.isArray(adminExtra.trustedSiteDirectives)
-    ) {
-        for ( const directive of adminExtra.trustedSiteDirectives ) {
-            µb.netWhitelistDefault.push(directive);
-            netWhitelistRaw.push(directive);
+
+    // Remove now obsolete built-in trusted directives
+    if ( lastVersionInt !== thisVersionInt ) {
+        if ( lastVersionInt < vAPI.app.intFromVersion('1.56.1b12') ) {
+            const obsolete = [
+                'about-scheme',
+                'chrome-scheme',
+                'edge-scheme',
+                'opera-scheme',
+                'vivaldi-scheme',
+                'wyciwyg-scheme',
+            ];
+            for ( const directive of obsolete ) {
+                const i = netWhitelistRaw.findIndex(s =>
+                    s === directive || s === `# ${directive}`
+                );
+                if ( i === -1 ) { continue; }
+                netWhitelistRaw.splice(i, 1);
+            }
         }
     }
+
+    // Append admin-controlled trusted-site directives
+    if ( adminExtra instanceof Object ) {
+        if ( Array.isArray(adminExtra.trustedSiteDirectives) ) {
+            for ( const directive of adminExtra.trustedSiteDirectives ) {
+                µb.netWhitelistDefault.push(directive);
+                netWhitelistRaw.push(directive);
+            }
+        }
+    }
+
     µb.netWhitelist = µb.whitelistFromArray(netWhitelistRaw);
     µb.netWhitelistModifyTime = Date.now();
 };
@@ -227,8 +254,7 @@ const onUserSettingsReady = fetched => {
         fetched.importedLists.length === 0 &&
         fetched.externalLists !== ''
     ) {
-        fetched.importedLists =
-            fetched.externalLists.trim().split(/[\n\r]+/);
+        fetched.importedLists = fetched.externalLists.trim().split(/[\n\r]+/);
     }
 
     fromFetch(µb.userSettings, fetched);
@@ -258,19 +284,19 @@ const onUserSettingsReady = fetched => {
 //   Wait for removal of invalid cached data to be completed.
 
 const onCacheSettingsReady = async (fetched = {}) => {
+    let selfieIsInvalid = false;
     if ( fetched.compiledMagic !== µb.systemSettings.compiledMagic ) {
         µb.compiledFormatChanged = true;
-        µb.selfieIsInvalid = true;
+        selfieIsInvalid = true;
         ubolog(`Serialized format of static filter lists changed`);
     }
     if ( fetched.selfieMagic !== µb.systemSettings.selfieMagic ) {
-        µb.selfieIsInvalid = true;
+        selfieIsInvalid = true;
         ubolog(`Serialized format of selfie changed`);
     }
-    if ( µb.selfieIsInvalid ) {
-        µb.selfieManager.destroy();
-        cacheStorage.set(µb.systemSettings);
-    }
+    if ( selfieIsInvalid === false ) { return; }
+    µb.selfieManager.destroy({ janitor: true });
+    cacheStorage.set(µb.systemSettings);
 };
 
 /******************************************************************************/
@@ -309,12 +335,6 @@ const onHiddenSettingsReady = async ( ) => {
             ubolog(`WASM modules ready ${Date.now()-vAPI.T0} ms after launch`);
         });
     }
-
-    // Maybe override default cache storage
-    const cacheBackend = await cacheStorage.select(
-        µb.hiddenSettings.cacheStorageAPI
-    );
-    ubolog(`Backend storage for cache will be ${cacheBackend}`);
 };
 
 /******************************************************************************/
@@ -328,7 +348,6 @@ const onFirstFetchReady = (fetched, adminExtra) => {
     }
 
     // Order is important -- do not change:
-    fromFetch(µb.localSettings, fetched);
     fromFetch(µb.restoreBackupSettings, fetched);
 
     permanentFirewall.fromString(fetched.dynamicFilteringString);
@@ -339,7 +358,6 @@ const onFirstFetchReady = (fetched, adminExtra) => {
     sessionSwitches.assign(permanentSwitches);
 
     onNetWhitelistReady(fetched.netWhitelist, adminExtra);
-    onVersionReady(fetched.version);
 };
 
 /******************************************************************************/
@@ -364,20 +382,18 @@ const createDefaultProps = ( ) => {
         'dynamicFilteringString': µb.dynamicFilteringDefault.join('\n'),
         'urlFilteringString': '',
         'hostnameSwitchesString': µb.hostnameSwitchesDefault.join('\n'),
-        'lastRestoreFile': '',
-        'lastRestoreTime': 0,
-        'lastBackupFile': '',
-        'lastBackupTime': 0,
         'netWhitelist': µb.netWhitelistDefault,
         'version': '0.0.0.0'
     };
-    toFetch(µb.localSettings, fetchableProps);
     toFetch(µb.restoreBackupSettings, fetchableProps);
     return fetchableProps;
 };
 
 /******************************************************************************/
 
+(async ( ) => {
+// >>>>> start of async/await scope
+    
 try {
     ubolog(`Start sequence of loading storage-based data ${Date.now()-vAPI.T0} ms after launch`);
 
@@ -392,23 +408,25 @@ try {
     const adminExtra = await vAPI.adminStorage.get('toAdd');
     ubolog(`Extra admin settings ready ${Date.now()-vAPI.T0} ms after launch`);
 
-    // https://github.com/uBlockOrigin/uBlock-issues/issues/1365
-    //   Wait for onCacheSettingsReady() to be fully ready.
-    const [ , , lastVersion ] = await Promise.all([
+    // Maybe override default cache storage
+    µb.supportStats.cacheBackend = await cacheStorage.select(
+        µb.hiddenSettings.cacheStorageAPI
+    );
+    ubolog(`Backend storage for cache will be ${µb.supportStats.cacheBackend}`);
+
+    await vAPI.storage.get(createDefaultProps()).then(async fetched => {
+        ubolog(`Version ready ${Date.now()-vAPI.T0} ms after launch`);
+        await onVersionReady(fetched.version);
+        return fetched;
+    }).then(fetched => {
+        ubolog(`First fetch ready ${Date.now()-vAPI.T0} ms after launch`);
+        onFirstFetchReady(fetched, adminExtra);
+    });
+
+    await Promise.all([
         µb.loadSelectedFilterLists().then(( ) => {
             ubolog(`List selection ready ${Date.now()-vAPI.T0} ms after launch`);
         }),
-        cacheStorage.get(
-            { compiledMagic: 0, selfieMagic: 0 }
-        ).then(fetched => {
-            ubolog(`Cache magic numbers ready ${Date.now()-vAPI.T0} ms after launch`);
-            onCacheSettingsReady(fetched);
-        }),
-        vAPI.storage.get(createDefaultProps()).then(fetched => {
-            ubolog(`First fetch ready ${Date.now()-vAPI.T0} ms after launch`);
-            onFirstFetchReady(fetched, adminExtra);
-            return fetched.version;
-        }),
         µb.loadUserSettings().then(fetched => {
             ubolog(`User settings ready ${Date.now()-vAPI.T0} ms after launch`);
             onUserSettingsReady(fetched);
@@ -416,10 +434,15 @@ try {
         µb.loadPublicSuffixList().then(( ) => {
             ubolog(`PSL ready ${Date.now()-vAPI.T0} ms after launch`);
         }),
+        cacheStorage.get({ compiledMagic: 0, selfieMagic: 0 }).then(bin => {
+            ubolog(`Cache magic numbers ready ${Date.now()-vAPI.T0} ms after launch`);
+            onCacheSettingsReady(bin);
+        }),
+        µb.loadLocalSettings(),
     ]);
 
     // https://github.com/uBlockOrigin/uBlock-issues/issues/1547
-    if ( lastVersion === '0.0.0.0' && vAPI.webextFlavor.soup.has('chromium') ) {
+    if ( lastVersionInt === 0 && vAPI.webextFlavor.soup.has('chromium') ) {
         vAPI.app.restart();
         return;
     }
@@ -437,7 +460,7 @@ let selfieIsValid = false;
 try {
     selfieIsValid = await µb.selfieManager.load();
     if ( selfieIsValid === true ) {
-        ubolog(`Selfie ready ${Date.now()-vAPI.T0} ms after launch`);
+        ubolog(`Loaded filtering engine from selfie ${Date.now()-vAPI.T0} ms after launch`);
     }
 } catch (ex) {
     console.trace(ex);
@@ -453,7 +476,7 @@ if ( selfieIsValid !== true ) {
 
 // Flush memory cache -- unsure whether the browser does this internally
 // when loading a new extension.
-µb.filteringBehaviorChanged();
+filteringBehaviorChanged();
 
 // Final initialization steps after all needed assets are in memory.
 
@@ -474,15 +497,6 @@ webRequest.start();
 // as possible ensure minimal memory usage baseline.
 lz4Codec.relinquish();
 
-// https://github.com/chrisaljoudi/uBlock/issues/184
-//   Check for updates not too far in the future.
-io.addObserver(µb.assetObserver.bind(µb));
-µb.scheduleAssetUpdater(
-    µb.userSettings.autoUpdate
-        ? µb.hiddenSettings.autoUpdateDelayAfterLaunch * 1000
-        : 0
-);
-
 // Force an update of the context menu according to the currently
 // active tab.
 contextMenu.update();
@@ -505,5 +519,49 @@ if ( selfieIsValid ) {
 }
 ubolog(`All ready ${µb.supportStats.allReadyAfter} after launch`);
 
-// <<<<< end of private scope
+µb.isReadyResolve();
+
+
+// https://github.com/chrisaljoudi/uBlock/issues/184
+//   Check for updates not too far in the future.
+io.addObserver(µb.assetObserver.bind(µb));
+if ( µb.userSettings.autoUpdate ) {
+    let needEmergencyUpdate = false;
+    const entries = await io.getUpdateAges({
+        filters: µb.selectedFilterLists,
+        internal: [ '*' ],
+    });
+    for ( const entry of entries ) {
+        if ( entry.ageNormalized < 2 ) { continue; }
+        needEmergencyUpdate = true;
+        break;
+    }
+    const updateDelay = needEmergencyUpdate
+        ? 2000
+        : µb.hiddenSettings.autoUpdateDelayAfterLaunch * 1000;
+    µb.scheduleAssetUpdater({
+        auto: true,
+        updateDelay,
+        fetchDelay: needEmergencyUpdate ? 1000 : undefined
+    });
+}
+
+// Process alarm queue
+while ( µb.alarmQueue.length !== 0 ) {
+    const what = µb.alarmQueue.shift();
+    ubolog(`Processing alarm event from suspended state: '${what}'`);
+    switch ( what ) {
+    case 'assetUpdater':
+        µb.scheduleAssetUpdater({ auto: true, updateDelay: 2000, fetchDelay : 1000 });
+        break;
+    case 'createSelfie':
+        µb.selfieManager.create();
+        break;
+    case 'saveLocalSettings':
+        µb.saveLocalSettings();
+        break;
+    }
+}
+
+// <<<<< end of async/await scope
 })();
